@@ -3,7 +3,7 @@
  * Ruby Cairo Binding
  *
  * $Author: kou $
- * $Date: 2007/05/20 23:31:57 $
+ * $Date: 2008-01-11 12:19:44 $
  *
  * Copyright 2005 Øyvind Kolås <pippin@freedesktop.org>
  * Copyright 2004-2005 MenTaLguY <mental@rydia.com>
@@ -26,10 +26,11 @@
 
 VALUE rb_cCairo_Surface;
 VALUE rb_cCairo_ImageSurface;
-VALUE rb_cCairo_PDFSurface;
-VALUE rb_cCairo_PSSurface;
-VALUE rb_cCairo_SVGSurface;
-VALUE rb_cCairo_WIN32Surface;
+VALUE rb_cCairo_PDFSurface = Qnil;
+VALUE rb_cCairo_PSSurface = Qnil;
+VALUE rb_cCairo_SVGSurface = Qnil;
+VALUE rb_cCairo_Win32Surface = Qnil;
+VALUE rb_cCairo_QuartzSurface = Qnil;
 
 static ID cr_id_target;
 static ID cr_id_read;
@@ -66,7 +67,7 @@ cr_surface_get_klass (cairo_surface_t *surface)
       klass = rb_cCairo_SVGSurface;
       break;
     case CAIRO_SURFACE_TYPE_WIN32:
-      klass = rb_cCairo_WIN32Surface;
+      klass = rb_cCairo_Win32Surface;
       break;
     default:
       rb_raise (rb_eArgError, "unknown source type: %d", type);
@@ -115,7 +116,7 @@ cr_surface_io_func_rescue (VALUE io_closure)
 {
   cr_io_callback_closure_t *closure;
   closure = (cr_io_callback_closure_t *)io_closure;
-  closure->error = ruby_errinfo;
+  closure->error = RB_ERRINFO;
   return Qnil;
 }
 
@@ -133,13 +134,13 @@ cr_surface_write_func_invoke (VALUE write_closure)
   output = closure->target;
   data = rb_str_new ((const char *)closure->data, closure->length);
 
-  length = RSTRING (data)->len;
+  length = RSTRING_LEN (data);
   while (length != 0)
     {
       VALUE rb_written_bytes = rb_funcall (output, cr_id_write, 1, data);
       written_bytes = NUM2LONG (rb_written_bytes);
       data = rb_str_substr (data, written_bytes,
-                            RSTRING (data)->len - written_bytes);
+                            RSTRING_LEN (data) - written_bytes);
       length -= written_bytes;
     }
   
@@ -180,7 +181,7 @@ cr_surface_read_func_invoke (VALUE read_closure)
   
   result = rb_str_new2 ("");
 
-  for (rest = length; rest != 0; rest = length - RSTRING (result)->len)
+  for (rest = length; rest != 0; rest = length - RSTRING_LEN (result))
     {
       rb_str_concat (result, rb_funcall (input, cr_id_read, 1, INT2NUM (rest)));
     }
@@ -830,6 +831,75 @@ cr_win32_surface_get_image (VALUE self)
 }
 #endif
 
+#if CAIRO_HAS_QUARTZ_SURFACE
+/* Quartz-surface functions */
+
+#include <objc/objc-runtime.h>
+
+VALUE ocobj_s_new (id ocid);
+id rbobj_get_ocid (VALUE obj);
+
+
+static VALUE
+cr_quartz_surface_initialize (int argc, VALUE *argv, VALUE self)
+{
+  id objc_object = nil;
+  CGContextRef context;
+  unsigned int width, height;
+  cairo_surface_t *surface = NULL;
+  cairo_format_t format = CAIRO_FORMAT_ARGB32;
+  VALUE arg1, arg2, arg3, rb_width, rb_height;
+
+  rb_scan_args (argc, argv, "21", &arg1, &arg2, &arg3);
+
+  if (argc == 2)
+    {
+      rb_width = arg1;
+      rb_height = arg2;
+    }
+  else
+    {
+      objc_object = rbobj_get_ocid (arg1);
+      if (objc_object == nil)
+        format = RVAL2CRFORMAT (arg1);
+
+      rb_width = arg2;
+      rb_height = arg3;
+    }
+
+  width = NUM2UINT (rb_width);
+  height = NUM2UINT (rb_height);
+
+  if (objc_object == nil)
+    {
+      surface = cairo_quartz_surface_create (format, width, height);
+    }
+  else
+    {
+      context = (CGContextRef)objc_object;
+      surface =
+        cairo_quartz_surface_create_for_cg_context (context, width, height);
+    }
+
+  cr_surface_check_status (surface);
+  DATA_PTR (self) = surface;
+  if (rb_block_given_p ())
+    yield_and_finish (self);
+  return Qnil;
+}
+
+static VALUE
+cr_quartz_surface_get_cg_context (VALUE self)
+{
+  CGContextRef context;
+  id objc_object;
+
+  context = cairo_quartz_surface_get_cg_context (_SELF);
+  objc_object = (id)context;
+  return ocobj_s_new (objc_object);
+}
+#endif
+
 
 void
 Init_cairo_surface (void)
@@ -909,8 +979,6 @@ Init_cairo_surface (void)
                     cr_ps_surface_dsc_begin_page_setup, 0);
 
   RB_CAIRO_DEF_SETTERS (rb_cCairo_PSSurface);
-#else
-  rb_cCairo_PSSurface = Qnil;
 #endif
 
 #if CAIRO_HAS_PDF_SURFACE
@@ -921,8 +989,6 @@ Init_cairo_surface (void)
                     cr_pdf_surface_set_size, 2);
 
   RB_CAIRO_DEF_SETTERS (rb_cCairo_PDFSurface);
-#else
-  rb_cCairo_PDFSurface = Qnil;
 #endif
 
 #if CAIRO_HAS_SVG_SURFACE
@@ -938,22 +1004,30 @@ Init_cairo_surface (void)
                     cr_svg_surface_restrict_to_version, 1);
 
   RB_CAIRO_DEF_SETTERS (rb_cCairo_SVGSurface);
-#else
-  rb_cCairo_SVGSurface = Qnil;
 #endif
 
 #if CAIRO_HAS_WIN32_SURFACE
-  /* WIN32-surface */
-  rb_cCairo_WIN32Surface =
-    rb_define_class_under (rb_mCairo, "WIN32Surface", rb_cCairo_Surface);
+  /* Win32-surface */
+  rb_cCairo_Win32Surface =
+    rb_define_class_under (rb_mCairo, "Win32Surface", rb_cCairo_Surface);
 
-  rb_define_method (rb_cCairo_WIN32Surface, "initialize",
+  rb_define_method (rb_cCairo_Win32Surface, "initialize",
                     cr_win32_surface_initialize, -1);
-  rb_define_method (rb_cCairo_WIN32Surface, "hdc",
+  rb_define_method (rb_cCairo_Win32Surface, "hdc",
                     cr_win32_surface_get_hdc, 0);
-  rb_define_method (rb_cCairo_WIN32Surface, "image",
+  rb_define_method (rb_cCairo_Win32Surface, "image",
                     cr_win32_surface_get_image, 0);
-#else
-  rb_cCairo_WIN32Surface = Qnil;
+#endif
+
+#if CAIRO_HAS_QUARTZ_SURFACE
+  /* Quartz-surface */
+
+  rb_cCairo_QuartzSurface =
+    rb_define_class_under (rb_mCairo, "QuartzSurface", rb_cCairo_Surface);
+
+  rb_define_method (rb_cCairo_QuartzSurface, "initialize",
+                    cr_quartz_surface_initialize, -1);
+  rb_define_method (rb_cCairo_QuartzSurface, "cg_context",
+                    cr_quartz_surface_get_cg_context, 0);
 #endif
 }
